"""Const for GPSLogger."""

DOMAIN = "gpslogger"

ATTR_ALTITUDE = "altitude"
ATTR_ACCURACY = "accuracy"
ATTR_ACTIVITY = "activity"
ATTR_DEVICE = "device"
ATTR_DIRECTION = "direction"
ATTR_PROVIDER = "provider"
ATTR_SPEED = "speed"
