"""Constants used by multiple MQTT modules."""
CONF_BROKER = "broker"
CONF_DISCOVERY = "discovery"
DEFAULT_DISCOVERY = False

ATTR_DISCOVERY_HASH = "discovery_hash"
CONF_STATE_TOPIC = "state_topic"
PROTOCOL_311 = "3.1.1"
DEFAULT_QOS = 0
